@@ -66,6 +66,13 @@ class Abort(Exception):
 class CommandLineInterface(object):
     """
     Wrapper around all the other classes, tying everything together.
+
+    Typical usage::
+
+        cli = CommandLineInterface()
+        while True:
+            result = cli.read_input()
+            print(result)
     """
     # All the factories below have to be callables that return instances of the
     # respective classes. They can be as simple as the class itself, but they
@@ -82,10 +89,6 @@ class CommandLineInterface(object):
     #: `Prompt` class for the layout of the prompt. (and the help text.)
     prompt_factory = Prompt
 
-    #: `InputStream` class for the parser of the input
-    #: (Normally, you don't override this one.)
-    inputstream_factory = InputStream
-
     #: `InputStreamHandler` class for the keybindings.
     key_bindings_factories = [ emacs_bindings ]
     default_input_mode = InputMode.INSERT
@@ -109,24 +112,16 @@ class CommandLineInterface(object):
         self.stdout = stdout or sys.stdout
 
         #: The `Line` instance.
-                    # TODO: construct the `Line` object outside of this constructor.
-                    #       A commandline should be able to have several `Line` instances.
-                    #       e.g. one for the incremental-search input, one for
-                    #       another document, etc...
-        self.line = self.line_factory(
-                                code_factory=self.code_factory,
-                                history_factory=self.history_factory,
-                                callbacks=LineCallbacks(self))
-
-        key_registry = Registry()
-        for kb in self.key_bindings_factories:
-            kb(key_registry, weakref.ref(self))
+        self.lines = self.create_lines()
 
         #: The `InputProcessor` instance.
-        self.input_processor = InputProcessor(key_registry, default_input_mode=self.default_input_mode)
+        self.input_processor = self.create_input_processor()
+
+        # Create `InputStream` instance.
+        self.inputstream = self.create_inputstream(self.input_processor, self.stdout)
 
         #: The `prompt` instance.
-        self.prompt = self.prompt_factory(weakref.ref(self))
+        self.prompt = self.create_prompt()
 
         #: The `Renderer` instance.
         self.renderer = self.renderer_factory(
@@ -147,6 +142,51 @@ class CommandLineInterface(object):
 
         self._reset()
 
+    @property
+    def line(self):
+        return self.lines['default']
+
+    def create_line_factories_dict(self):
+        """
+        Return declarative structure of the Line objects to be created.
+        """
+        return {
+            'default': (self.line_factory, self.code_factory, self.history_factory()),
+            'search': (Line, Code, History()),
+        }
+
+    def create_lines(self):
+        """
+        Create :class:`Line` instances.
+
+        We can have several `Line` instances. E.g: one for the main document,
+        one for the incremental-search input, etc...
+        """
+        def _create_line(line_cls, code_cls, history):
+            return line_cls(code_factory=code_cls, history=history, callbacks=LineCallbacks(self))
+
+        return { name: _create_line(*k) for (name, k) in self.create_line_factories_dict().items() }
+
+    def create_inputstream(self, input_processor, stdout):
+        return InputStream(input_processor, stdout=stdout)
+
+    def create_input_processor(self):
+        """
+        Create :class:`InputProcessor` instance.
+        """
+        key_registry = Registry()
+        for kb in self.key_bindings_factories:
+            kb(key_registry, weakref.ref(self))
+
+        #: The `InputProcessor` instance.
+        return InputProcessor(key_registry, default_input_mode=self.default_input_mode)
+
+    def create_prompt(self):
+        """
+        Create :class:`Prompt` instance.
+        """
+        return self.prompt_factory(weakref.ref(self))
+
     def _reset(self):
         self._exit_flag = False
         self._abort_flag = False
@@ -197,7 +237,7 @@ class CommandLineInterface(object):
         """
         The input 'event loop'.
 
-        This should return the next character to process.
+        This should return the next characters to process.
         """
         timeout = self.input_timeout
 
@@ -261,11 +301,9 @@ class CommandLineInterface(object):
             self._redraw_pipe = os.pipe()
             fcntl.fcntl(self._redraw_pipe[0], fcntl.F_SETFL, os.O_NONBLOCK)
 
-            stream = self.inputstream_factory(self.input_processor, stdout=self.stdout)
-
             def reset_line():
                 """ Reset everything. """
-                stream.reset()
+                self.inputstream.reset()
                 self.line.reset(initial_value=initial_value)
                 self.renderer.reset()
                 self.input_processor.reset()
@@ -290,7 +328,10 @@ class CommandLineInterface(object):
                         # got none, it means we got a repaint request.
                         if c:
                             # Feed input text.
-                            stream.feed(c)
+                            self.inputstream.feed(c)
+
+                            # Immediately flush the input.
+                            self.inputstream.flush()
 
                         # If the exit flag has been set.
                         if self._exit_flag:
