@@ -298,12 +298,18 @@ class PythonPrompt(Prompt):
         """
         When inside functions, show signature.
         """
-        if self.commandline.input_processor.input_mode == InputMode.INCREMENTAL_SEARCH and self.line.isearch_state:
+        if self.commandline.input_processor.input_mode == InputMode.VI_SEARCH:
+            self.write_vi_search(screen)
+
+        elif self.commandline.input_processor.input_mode == InputMode.INCREMENTAL_SEARCH and self.line.isearch_state:
             screen.write_highlighted(list(self.isearch_prompt))
+
         elif self.commandline.input_processor.arg is not None:
             screen.write_highlighted(list(self.arg_prompt))
+
         elif self.line.validation_error:
             screen.write_highlighted(list(self._get_error_tokens()))
+
         elif self.commandline.autocompletion_style == AutoCompletionStyle.HORIZONTAL_MENU and \
                         self.line.complete_state and \
                         self.commandline.input_processor.input_mode == InputMode.COMPLETE:
@@ -383,7 +389,6 @@ class PythonPrompt(Prompt):
             y2 = self._get_bottom_position(screen, last_screen_height)
             y = max(y, y2 - 1)
 
-
             screen._y, screen._x = y + 1, 0
             self.write_second_toolbar(screen)
 
@@ -409,6 +414,9 @@ class PythonPrompt(Prompt):
             if mode == InputMode.INSERT:
                 append((TB.Mode, '(INSERT)'))
                 append((TB, '   '))
+            elif mode == InputMode.VI_SEARCH:
+                append((TB.Mode, '(SEARCH)'))
+                append((TB, '   '))
             elif mode == InputMode.VI_NAVIGATION:
                 append((TB.Mode, '(NAV)'))
                 append((TB, '      '))
@@ -418,7 +426,7 @@ class PythonPrompt(Prompt):
             elif mode == InputMode.COMPLETE:
                 append((TB.Mode, '(COMPLETE)'))
                 append((TB, ' '))
-            elif mode == InputMode.SELECTION:
+            elif mode == InputMode.SELECTION and self.line.selection_state:
                 if self.line.selection_state.type == SelectionType.LINES:
                     append((TB.Mode, '(VISUAL LINE)'))
                     append((TB, ' '))
@@ -435,12 +443,12 @@ class PythonPrompt(Prompt):
             append((TB, ' '))
 
         # Position in history.
-        append((TB, '%i/%i ' % (self.line._working_index + 1, len(self.line._working_lines))))
+        append((TB, '%i/%i ' % (self.line.working_index + 1, len(self.line._working_lines))))
 
         # Shortcuts.
         if mode == InputMode.INCREMENTAL_SEARCH:
             append((TB, '[Ctrl-G] Cancel search [Enter] Go to this position.'))
-        elif self.line.selection_state:
+        elif mode == InputMode.SELECTION and not self.commandline.vi_mode:
             # Emacs cut/copy keys.
             append((TB, '[Ctrl-W] Cut [Meta-W] Copy [Ctrl-Y] Paste [Ctrl-G] Cancel'))
         else:
@@ -596,10 +604,10 @@ class PythonCommandLineInterface(CommandLineInterface):
             return
         self.get_signatures_thread_running = True
 
-        code_obj = self.line.create_code_obj()
+        code = self.line.create_code()
 
         def run():
-            script = code_obj._get_jedi_script()
+            script = code._get_jedi_script()
 
             # Show signatures in help text.
             if script:
@@ -619,7 +627,7 @@ class PythonCommandLineInterface(CommandLineInterface):
 
             # Set signatures and redraw if the text didn't change in the
             # meantime. Otherwise request new signatures.
-            if self.line.text == code_obj.text:
+            if self.line.text == code.text:
                 self.line.signatures = signatures
                 self.request_redraw()
             else:
