@@ -1,7 +1,7 @@
 from __future__ import unicode_literals
 
 from ..keys import Keys
-from ..enums import InputMode, IncrementalSearchDirection
+from ..enums import InputMode
 from ..line import ClipboardData
 
 from .utils import create_handle_decorator
@@ -56,24 +56,18 @@ def basic_bindings(registry, cli_ref):
         # We override the functionality below.
         pass
 
-    @handle(Keys.Home)
+    @handle(Keys.Home, in_mode=InputMode.INSERT)
+    @handle(Keys.Home, in_mode=InputMode.SELECTION)
     def _(event):
         line.cursor_position += line.document.home_position
 
-    @handle(Keys.End)
+    @handle(Keys.End, in_mode=InputMode.INSERT)
+    @handle(Keys.End, in_mode=InputMode.SELECTION)
     def _(event):
         line.cursor_position += line.document.end_position
 
     # CTRL keys.
 
-    @handle(Keys.ControlA)
-    def _(event):
-        line.cursor_position += line.document.get_start_of_line_position(after_whitespace=False)
-
-    @handle(Keys.ControlB)
-    def _(event):
-        line.cursor_position += line.document.get_cursor_left_position(count=event.arg)
-
     @handle(Keys.ControlC)
     def _(event):
         line.abort()
@@ -90,20 +84,6 @@ def basic_bindings(registry, cli_ref):
     def _(event):
         line.cursor_position += line.document.get_end_of_line_position()
 
-    @handle(Keys.ControlF)
-    def _(event):
-        line.cursor_position += line.document.get_cursor_right_position(count=event.arg)
-
-    @handle(Keys.ControlG, in_mode=InputMode.INCREMENTAL_SEARCH)
-    # NOTE: the reason for not binding Escape to this one, is that we want
-    #       Alt+Enter to accept input directly in incremental search mode.
-    def _(event):
-        """
-        Abort an incremental search and restore the original line.
-        """
-        line.exit_isearch(restore_original_line=True)
-        event.input_processor.pop_input_mode()
-
     @handle(Keys.ControlI, in_mode=InputMode.INSERT)
     @handle(Keys.ControlI, in_mode=InputMode.COMPLETE)
     def _(event):
@@ -119,16 +99,6 @@ def basic_bindings(registry, cli_ref):
         else:
             not line.complete_common()
 
-    @handle(Keys.ControlJ, in_mode=InputMode.INCREMENTAL_SEARCH)
-    @handle(Keys.ControlM, in_mode=InputMode.INCREMENTAL_SEARCH)
-    def _(event):
-        """
-        When enter pressed in isearch, quit isearch mode. (Multiline
-        isearch would be too complicated.)
-        """
-        line.exit_isearch()
-        event.input_processor.pop_input_mode()
-
     @handle(Keys.ControlJ, in_mode=InputMode.INSERT)
     @handle(Keys.ControlM, in_mode=InputMode.INSERT)
     def _(event):
@@ -149,22 +119,6 @@ def basic_bindings(registry, cli_ref):
     def _(event):
         line.clear()
 
-    @handle(Keys.ControlR)
-    @handle(Keys.Up, in_mode=InputMode.INCREMENTAL_SEARCH)
-    def _(event):
-        line.incremental_search(IncrementalSearchDirection.BACKWARD)
-
-        if event.input_processor.input_mode != InputMode.INCREMENTAL_SEARCH:
-            event.input_processor.push_input_mode(InputMode.INCREMENTAL_SEARCH)
-
-    @handle(Keys.ControlS)
-    @handle(Keys.Down, in_mode=InputMode.INCREMENTAL_SEARCH)
-    def _(event):
-        line.incremental_search(IncrementalSearchDirection.FORWARD)
-
-        if event.input_processor.input_mode != InputMode.INCREMENTAL_SEARCH:
-            event.input_processor.push_input_mode(InputMode.INCREMENTAL_SEARCH)
-
     @handle(Keys.ControlT)
     def _(event):
         line.swap_characters_before_cursor()
@@ -188,10 +142,6 @@ def basic_bindings(registry, cli_ref):
             deleted = line.delete_before_cursor(count=-pos)
             line.set_clipboard(ClipboardData(deleted))
 
-    @handle(Keys.ControlX)
-    def _(event):
-        pass
-
     @handle(Keys.ControlY, InputMode.INSERT)
     def _(event):
         # Pastes the clipboard content.
@@ -213,34 +163,45 @@ def basic_bindings(registry, cli_ref):
     def _(event):
         line.history_forward()
 
-    @handle(Keys.Left)
+    @handle(Keys.Left, in_mode=InputMode.INSERT)
+    @handle(Keys.Left, in_mode=InputMode.SELECTION)
     def _(event):
-        if not event.input_processor.input_mode == InputMode.INCREMENTAL_SEARCH:
-            line.cursor_position += line.document.get_cursor_left_position(count=event.arg)
+        line.cursor_position += line.document.get_cursor_left_position(count=event.arg)
 
-    @handle(Keys.Right)
+    @handle(Keys.Right, in_mode=InputMode.INSERT)
+    @handle(Keys.Right, in_mode=InputMode.SELECTION)
     def _(event):
-        if not event.input_processor.input_mode == InputMode.INCREMENTAL_SEARCH:
-            line.cursor_position += line.document.get_cursor_right_position(count=event.arg)
+        line.cursor_position += line.document.get_cursor_right_position(count=event.arg)
 
-    @handle(Keys.Up)
+    @handle(Keys.Up, in_mode=InputMode.INSERT)
     def _(event):
         line.auto_up(count=event.arg)
 
-    @handle(Keys.Down)
+    @handle(Keys.Down, in_mode=InputMode.INSERT)
     def _(event):
         line.auto_down(count=event.arg)
 
+    @handle(Keys.Up, in_mode=InputMode.SELECTION)
+    def _(event):
+        line.cursor_up(count=event.arg)
+
+    @handle(Keys.Down, in_mode=InputMode.SELECTION)
+    def _(event):
+        line.cursor_down(count=event.arg)
+
+    @handle(Keys.Up, in_mode=InputMode.COMPLETE)
+    def _(event):
+        line.complete_previous()
+
+    @handle(Keys.Down, in_mode=InputMode.COMPLETE)
+    def _(event):
+        line.complete_next()
+
     @handle(Keys.ControlH, in_mode=InputMode.INSERT)
     @handle(Keys.Backspace, in_mode=InputMode.INSERT)
     def _(event):
         line.delete_before_cursor(count=event.arg)
 
-    @handle(Keys.ControlH, in_mode=InputMode.INCREMENTAL_SEARCH)
-    @handle(Keys.Backspace, in_mode=InputMode.INCREMENTAL_SEARCH)
-    def _(event):
-        line.isearch_delete_before_cursor(count=event.arg)
-
     @handle(Keys.Delete, in_mode=InputMode.INSERT)
     def _(event):
         line.delete(count=event.arg)
@@ -249,13 +210,6 @@ def basic_bindings(registry, cli_ref):
     def _(event):
         line.delete(count=event.arg)
 
-    @handle(Keys.Any, in_mode=InputMode.INCREMENTAL_SEARCH)
-    def _(event):
-        """
-        Insert isearch string.
-        """
-        line.insert_isearch_text(event.data)
-
     @handle(Keys.Any, in_mode=InputMode.COMPLETE)
     @handle(Keys.Any, in_mode=InputMode.INSERT)
     def _(event):
@@ -268,20 +222,6 @@ def basic_bindings(registry, cli_ref):
         if event.input_processor.input_mode == InputMode.COMPLETE:
             event.input_processor.pop_input_mode()
 
-    @handle(Keys.Escape, Keys.Left)
-    def _(event):
-        """
-        Cursor to start of previous word.
-        """
-        line.cursor_position += line.document.find_previous_word_beginning(count=event.arg) or 0
-
-    @handle(Keys.Escape, Keys.Right)
-    def _(event):
-        """
-        Cursor to start of next word.
-        """
-        line.cursor_position += line.document.find_next_word_beginning(count=event.arg) or 0
-
     @handle(Keys.Escape, in_mode=InputMode.COMPLETE)
     @handle(Keys.ControlC, in_mode=InputMode.COMPLETE)
     def _(event):
@@ -289,4 +229,3 @@ def basic_bindings(registry, cli_ref):
         Pressing escape or Ctrl-C in complete mode, goes back to default mode.
         """
         event.input_processor.pop_input_mode()
-
