@@ -7,6 +7,8 @@ from ..keys import Keys
 from .basic import basic_bindings
 from .utils import create_handle_decorator
 
+import codecs
+
 __all__ = (
     'vi_bindings',
 )
@@ -20,7 +22,10 @@ class CursorRegion(object):
         self.start = start
         self.end = end
 
-    def get_sorted(self):
+    def sorted(self):
+        """
+        Return a (start, end) tuple where start <= end.
+        """
         if self.start < self.end:
             return self.start, self.end
         else:
@@ -36,14 +41,26 @@ def vi_bindings(registry, cli_ref):
     """
     basic_bindings(registry, cli_ref)
     line = cli_ref().line
+    search_line = cli_ref().lines['search']
     handle = create_handle_decorator(registry, line)
 
     _last_character_find = [None] # (char, backwards) tuple
+    _search_direction = [IncrementalSearchDirection.FORWARD]
 
-#    def reset(self):
-#        # Remember the last 'F' or 'f' command.
-#        self._last_character_find = None # (char, backwards) tuple
+    vi_transform_functions = [
+            # Rot 13 transformation
+            (('g', '?'), lambda string: codecs.encode(string, 'rot_13')),
 
+            # To lowercase
+            (('g', 'u'), lambda string: string.lower()),
+
+            # To uppercase.
+            (('g', 'U'), lambda string: string.upper()),
+
+            # Swap case.
+            # (XXX: If we would implement 'tildeop', the 'g' prefix is not required.)
+            (('g', '~'), lambda string: string.swapcase()),
+    ]
 
     @registry.add_after_handler_callback
     def check_cursor_position(event):
@@ -69,6 +86,20 @@ def vi_bindings(registry, cli_ref):
         else:
             event.input_processor.input_mode = InputMode.VI_NAVIGATION
 
+    @handle(Keys.Up, in_mode=InputMode.VI_NAVIGATION)
+    def _(event):
+        """
+        Arrow up in navigation mode.
+        """
+        line.auto_up(count=event.arg)
+
+    @handle(Keys.Down, in_mode=InputMode.VI_NAVIGATION)
+    def _(event):
+        """
+        Arrow down in navigation mode.
+        """
+        line.auto_down(count=event.arg)
+
     @handle(Keys.Backspace, in_mode=InputMode.VI_NAVIGATION)
     def _(event):
         """
@@ -185,10 +216,16 @@ def vi_bindings(registry, cli_ref):
     @handle('G', in_mode=InputMode.VI_NAVIGATION)
     def _(event):
         """
-        Move to the history line n (you may specify the argument n by
-        typing it on number keys, for example, 15G)
+        If an argument is given, move to this line in the  history. (for
+        example, 15G) Otherwise, go the the last line of the current string.
         """
-        line.go_to_history(event.arg - 1)
+        # If an arg has been given explicitely.
+        if event._arg:
+            line.go_to_history(event.arg - 1)
+
+        # Otherwise this goes to the last line of the file.
+        else:
+            line.cursor_position = len(line.text)
 
     @handle('i', in_mode=InputMode.VI_NAVIGATION)
     def _(event):
@@ -205,25 +242,21 @@ def vi_bindings(registry, cli_ref):
             line.join_next_line()
 
     @handle('n', in_mode=InputMode.VI_NAVIGATION)
-    def _(event): # XXX: use `change_delete_move_yank_handler`
-        # TODO:
-        pass
-
-        # if line.isearch_state:
-        #     # Repeat search in the same direction as previous.
-        #     line.search_next(line.isearch_state.isearch_direction)
+    def _(event): # XXX: use `change_delete_move_yank_handler` and implement 'arg'
+        """
+        Search next.
+        """
+        line.incremental_search(_search_direction[0])
 
     @handle('N', in_mode=InputMode.VI_NAVIGATION)
-    def _(event): # TODO: use `change_delete_move_yank_handler`
-        # TODO:
-        pass
-
-        #if line.isearch_state:
-        #    # Repeat search in the opposite direction as previous.
-        #    if line.isearch_state.isearch_direction == IncrementalSearchDirection.FORWARD:
-        #        line.search_next(IncrementalSearchDirection.BACKWARD)
-        #    else:
-        #        line.search_next(IncrementalSearchDirection.FORWARD)
+    def _(event): # TODO: use `change_delete_move_yank_handler` and implement 'arg'
+        """
+        Search previous.
+        """
+        if _search_direction[0] == IncrementalSearchDirection.BACKWARD:
+            line.incremental_search(IncrementalSearchDirection.FORWARD)
+        else:
+            line.incremental_search(IncrementalSearchDirection.BACKWARD)
 
     @handle('p', in_mode=InputMode.VI_NAVIGATION)
     def _(event):
@@ -309,6 +342,17 @@ def vi_bindings(registry, cli_ref):
         line.set_clipboard(ClipboardData(deleted, selection_type))
         event.input_processor.pop_input_mode()
 
+    @handle('c', in_mode=InputMode.SELECTION)
+    def _(event):
+        """
+        Change selection (cut and go to insert mode).
+        """
+        selection_type = line.selection_state.type
+        deleted = line.cut_selection()
+        line.set_clipboard(ClipboardData(deleted, selection_type))
+        event.input_processor.pop_input_mode()
+        event.input_processor.input_mode = InputMode.INSERT
+
     @handle('y', in_mode=InputMode.SELECTION)
     def _(event):
         """
@@ -325,6 +369,7 @@ def vi_bindings(registry, cli_ref):
         line.set_clipboard(data)
 
     @handle('y', 'y', in_mode=InputMode.VI_NAVIGATION)
+    @handle('Y', in_mode=InputMode.VI_NAVIGATION)
     def _(event):
         """
         Yank the whole line.
@@ -356,9 +401,25 @@ def vi_bindings(registry, cli_ref):
         Indent lines.
         """
         current_line = line.document.cursor_position_row
-        line_range = range(current_line, current_line + event.arg)
-        line.transform_lines(line_range, lambda l: '    ' + l)
+        _indent(current_line, current_line + event.arg)
 
+    def _indent(from_line, to_line, count=1):
+        line_range = range(from_line, to_line)
+        line.transform_lines(line_range, lambda l: '    ' * count + l)
+
+        line.cursor_position += line.document.get_start_of_line_position(after_whitespace=True)
+
+    def _unindent(from_line, to_line, count=1):
+        line_range = range(from_line, to_line)
+
+        def transform(text):
+            remove = '    ' * count
+            if text.startswith(remove):
+                return text[len(remove):]
+            else:
+                return text.lstrip()
+
+        line.transform_lines(line_range, transform)
         line.cursor_position += line.document.get_start_of_line_position(after_whitespace=True)
 
     @handle('<', '<', in_mode=InputMode.VI_NAVIGATION)
@@ -367,16 +428,35 @@ def vi_bindings(registry, cli_ref):
         Unindent lines.
         """
         current_line = line.document.cursor_position_row
-        line_range = range(current_line, current_line + event.arg)
+        _unindent(current_line, current_line + event.arg)
 
-        def transform(text):
-            if text.startswith('    '):
-                return text[4:]
-            else:
-                return text.lstrip()
+    @handle('>', in_mode=InputMode.SELECTION)
+    def _(event):
+        """
+        Indent selection
+        """
+        selection_type = line.selection_state.type
+        if selection_type == SelectionType.LINES:
+            from_, to = line.document.selection_range()
+            from_, _ = line.document.translate_index_to_position(from_)
+            to, _ = line.document.translate_index_to_position(to)
 
-        line.transform_lines(line_range, transform)
-        line.cursor_position += line.document.get_start_of_line_position(after_whitespace=True)
+            _indent(from_ - 1, to, count=event.arg) # XXX: why does translate_index_to_position return 1-based indexing???
+        event.input_processor.pop_input_mode()
+
+    @handle('<', in_mode=InputMode.SELECTION)
+    def _(event):
+        """
+        Unindent selection
+        """
+        selection_type = line.selection_state.type
+        if selection_type == SelectionType.LINES:
+            from_, to = line.document.selection_range()
+            from_, _ = line.document.translate_index_to_position(from_)
+            to, _ = line.document.translate_index_to_position(to)
+
+            _unindent(from_ - 1, to, count=event.arg)
+        event.input_processor.pop_input_mode()
 
     @handle('O', in_mode=InputMode.VI_NAVIGATION)
     def _(event):
@@ -407,18 +487,20 @@ def vi_bindings(registry, cli_ref):
     @handle('/', in_mode=InputMode.VI_NAVIGATION)
     def _(event):
         """
-        Search history backward for a command matching string.
+        Vi-style forward search.
         """
-        line.incremental_search(IncrementalSearchDirection.FORWARD)
-        event.input_processor.push_input_mode(InputMode.INCREMENTAL_SEARCH)
+        line.start_isearch(IncrementalSearchDirection.FORWARD)
+        event.input_processor.push_input_mode(InputMode.VI_SEARCH)
+        _search_direction[0] = IncrementalSearchDirection.FORWARD
 
     @handle('?', in_mode=InputMode.VI_NAVIGATION)
     def _(event):
         """
-        Search history forward for a command matching string.
+        Vi-style backward search.
         """
-        line.incremental_search(IncrementalSearchDirection.BACKWARD)
-        event.input_processor.push_input_mode(InputMode.INCREMENTAL_SEARCH)
+        line.start_isearch(IncrementalSearchDirection.BACKWARD)
+        event.input_processor.push_input_mode(InputMode.VI_SEARCH)
+        _search_direction[0] = IncrementalSearchDirection.BACKWARD
 
     @handle('#', in_mode=InputMode.VI_NAVIGATION)
     def _(event):
@@ -454,6 +536,8 @@ def vi_bindings(registry, cli_ref):
         no_move_handler = kw.pop('no_move_handler', False)
 
         # TODO: Also do '>' and '<' indent/unindent operators.
+        # TODO: Also "gq": text formatting
+        #  See: :help motion.txt
         def decorator(func):
             if not no_move_handler:
                 @handle(*keys, in_mode=InputMode.VI_NAVIGATION)
@@ -463,12 +547,31 @@ def vi_bindings(registry, cli_ref):
                     region = func(event)
                     line.cursor_position += region.start
 
+            def create_transform_handler(transform_func, *a):
+                @handle(*(a + keys), in_mode=InputMode.VI_NAVIGATION)
+                def _(event):
+                    """ Apply transformation (uppercase, lowercase, rot13, swap case). """
+                    region = func(event)
+                    start, end = region.sorted()
+
+                    # Transform.
+                    line.transform_region(
+                                line.cursor_position + start,
+                                line.cursor_position + end,
+                                transform_func)
+
+                    # Move cursor
+                    line.cursor_position += (region.end or region.start)
+
+            for k, f in vi_transform_functions:
+                create_transform_handler(f, *k)
+
             @handle('y', *keys, in_mode=InputMode.VI_NAVIGATION)
             def yank_handler(event):
                 """ Create yank handler. """
                 region = func(event)
 
-                start, end = region.get_sorted()
+                start, end = region.sorted()
                 substring = line.text[line.cursor_position + start : line.cursor_position + end]
 
                 if substring:
@@ -483,7 +586,7 @@ def vi_bindings(registry, cli_ref):
                     deleted = ''
 
                     if region:
-                        start, end = region.get_sorted()
+                        start, end = region.sorted()
 
                         # Move to the start of the region.
                         line.cursor_position += start
@@ -659,6 +762,7 @@ def vi_bindings(registry, cli_ref):
     repeat(False)
 
     @change_delete_move_yank_handler('h')
+    @change_delete_move_yank_handler(Keys.Left)
     def _(event):
         """ Implements 'ch', 'dh', 'h': Cursor left. """
         return CursorRegion(line.document.get_cursor_left_position(count=event.arg))
@@ -675,6 +779,7 @@ def vi_bindings(registry, cli_ref):
 
     @change_delete_move_yank_handler('l')
     @change_delete_move_yank_handler(' ')
+    @change_delete_move_yank_handler(Keys.Right)
     def _(event):
         """ Implements 'cl', 'dl', 'l', 'c ', 'd ', ' '. Cursor right. """
         return CursorRegion(line.document.get_cursor_right_position(count=event.arg))
@@ -694,9 +799,23 @@ def vi_bindings(registry, cli_ref):
 
     @change_delete_move_yank_handler('%')
     def _(event):
-        """ Implements 'c%', 'd%', '%, 'y%' """
-        # Move to the corresponding opening/closing bracket (()'s, []'s and {}'s).
-        return CursorRegion(line.document.matching_bracket_position)
+        """
+        Implements 'c%', 'd%', '%, 'y%' (Move to corresponding bracket.)
+        If an 'arg' has been given, go this this % position in the file.
+        """
+        if event._arg:
+            # If 'arg' has been given, the meaning of % is to go to the 'x%'
+            # row in the file.
+            if 0 < event.arg <= 100:
+                absolute_index = line.document.translate_row_col_to_index(
+                                    int(event.arg * line.document.line_count / 100), 0)
+                return CursorRegion(absolute_index - line.document.cursor_position)
+            else:
+                return CursorRegion(0) # Do nothing.
+
+        else:
+            # Move to the corresponding opening/closing bracket (()'s, []'s and {}'s).
+            return CursorRegion(line.document.matching_bracket_position)
 
     @change_delete_move_yank_handler('|')
     def _(event):
@@ -729,3 +848,93 @@ def vi_bindings(registry, cli_ref):
         Insert data at cursor position.
         """
         line.insert_text(event.data, overwrite=True)
+
+    @handle(Keys.Any, in_mode=InputMode.VI_SEARCH)
+    def _(event):
+        """
+        Insert text after the / or ? prompt.
+        """
+        search_line.insert_text(event.data)
+        line.set_search_text(search_line.text)
+
+    @handle(Keys.ControlJ, in_mode=InputMode.VI_SEARCH)
+    @handle(Keys.ControlM, in_mode=InputMode.VI_SEARCH)
+    def _(event):
+        """
+        Enter at the / or ? prompt.
+        """
+        # Call return_input to put the current result on the history stack
+        # of search operations.
+        search_line.return_input(call_callback=False)
+        search_line.reset()
+
+        # Go back to navigation mode.
+        event.input_processor.pop_input_mode()
+
+    @handle(Keys.Backspace, in_mode=InputMode.VI_SEARCH)
+    def _(event):
+        """
+        Backspace at the vi-search prompt.
+        """
+        if search_line.text:
+            search_line.delete_before_cursor()
+            line.set_search_text(search_line.text)
+        else:
+            # If no text after the prompt, cancel search.
+            line.exit_isearch(restore_original_line=True)
+            search_line.reset()
+            event.input_processor.pop_input_mode()
+
+    @handle(Keys.Up, in_mode=InputMode.VI_SEARCH)
+    def _(event):
+        """
+        Go to the previous history item at the search prompt.
+        """
+        search_line.auto_up()
+        line.set_search_text(search_line.text)
+
+    @handle(Keys.Down, in_mode=InputMode.VI_SEARCH)
+    def _(event):
+        """
+        Go to the next history item at the search prompt.
+        """
+        search_line.auto_down()
+        search_line.cursor_position = len(search_line.text)
+        line.set_search_text(search_line.text)
+
+    @handle(Keys.Left, in_mode=InputMode.VI_SEARCH)
+    def _(event):
+        """
+        Arrow left at the search prompt.
+        """
+        search_line.cursor_left()
+
+    @handle(Keys.Right, in_mode=InputMode.VI_SEARCH)
+    def _(event):
+        """
+        Arrow right at the search prompt.
+        """
+        search_line.cursor_right()
+
+    @handle(Keys.ControlC, in_mode=InputMode.VI_SEARCH)
+    def _(event):
+        """
+        Cancel search.
+        """
+        line.exit_isearch(restore_original_line=True)
+        search_line.reset()
+        event.input_processor.pop_input_mode()
+
+    def create_selection_transform_handler(keys, transform_func):
+        """
+        Apply transformation on selection (uppercase, lowercase, rot13, swap case).
+        """
+        @handle(*keys, in_mode=InputMode.SELECTION)
+        def _(event):
+            range = line.document.selection_range()
+            if range:
+                line.transform_region(range[0], range[1], transform_func)
+            event.input_processor.pop_input_mode()
+
+    for k, f in vi_transform_functions:
+        create_selection_transform_handler(k, f)
