@@ -99,7 +99,7 @@ class CompletionState(object):
 
 
 class _IncrementalSearchState(object):
-    def __init__(self, original_cursor_position, original_working_index):
+    def __init__(self, original_cursor_position, original_working_index, direction):
         self.isearch_text = ''
 
         self.original_working_index = original_working_index
@@ -109,7 +109,7 @@ class _IncrementalSearchState(object):
         #: This flag is updated every time we search for a new string.
         self.no_match_from_index = None
 
-        self.isearch_direction = IncrementalSearchDirection.FORWARD
+        self.isearch_direction = direction
 
 
 class Line(object):
@@ -131,11 +131,11 @@ class Line(object):
     #: Suffix to be appended to the tempfile for the 'open in editor' function.
     tempfile_suffix = ''
 
-    def __init__(self, code_factory=Code, history_factory=History, callbacks=None):
+    def __init__(self, code_factory=Code, history=None, callbacks=None):
         self.code_factory = code_factory
 
         #: The command line history.
-        self._history = history_factory()
+        self._history = history or History()
 
         self._callbacks = callbacks or Callbacks()
 
@@ -176,11 +176,11 @@ class Line(object):
 
     @property
     def text(self):
-        return self._working_lines[self._working_index]
+        return self._working_lines[self.working_index]
 
     @text.setter
     def text(self, value):
-        self._working_lines[self._working_index] = value
+        self._working_lines[self.working_index] = value
 
         # Remove validation errors, complete state.
         self.validation_error = None
@@ -205,11 +205,11 @@ class Line(object):
         # new position of the cursor determines the end of the selection.
 
     @property
-    def _working_index(self):
+    def working_index(self):
         return self.__working_index
 
-    @_working_index.setter
-    def _working_index(self, value):
+    @working_index.setter
+    def working_index(self, value):
         self.__working_index = value
 
         # Remove any validation errors and complete state.
@@ -266,6 +266,23 @@ class Line(object):
 
         self.text = '\n'.join(lines)
 
+    def transform_region(self, from_, to, transform_callback):
+        """
+        Transform a part of the input string.
+
+        :param :from_: (int) start position.
+        :param :to: (int) end position.
+        :param :transform_callback: Callable which accepts a string and returns
+                                    the transformed string.
+        """
+        assert from_ < to
+
+        self.text = ''.join([
+                    self.text[:from_] +
+                    transform_callback(self.text[from_:to]) +
+                    self.text[to:]
+                ])
+
     @property
     def document(self):
         """
@@ -275,28 +292,19 @@ class Line(object):
         # TODO: this can be cached as long self.text does not change.
         return Document(self.text, self.cursor_position, selection=self.selection_state)
 
-    def set_arg_prompt(self, arg): # XXX: Just make a property of this???
-        """
-        Called from the `InputStreamHandler` to set a "(arg: x)"-like prompt.
-        (Both in Vi and Emacs-mode we have a way to repeat line operations.
-        Settings this attribute to the `Line` object allows the prompt/renderer
-        to visualise it.)
-        """
-        self._arg_prompt_text = arg
+    def cursor_left(self, count=1):
+        self.cursor_position += self.document.get_cursor_left_position(count=count)
 
-    def cursor_left(self):
-        self.cursor_position += self.document.get_cursor_left_position()
+    def cursor_right(self, count=1):
+        self.cursor_position += self.document.get_cursor_right_position(count=count)
 
-    def cursor_right(self):
-        self.cursor_position += self.document.get_cursor_right_position()
-
-    def cursor_up(self):
+    def cursor_up(self, count=1):
         """ (for multiline edit). Move cursor to the previous line.  """
-        self.cursor_position += self.document.get_cursor_up_position()
+        self.cursor_position += self.document.get_cursor_up_position(count=count)
 
-    def cursor_down(self):
+    def cursor_down(self, count=1):
         """ (for multiline edit). Move cursor to the next line.  """
-        self.cursor_position += self.document.get_cursor_down_position()
+        self.cursor_position += self.document.get_cursor_down_position(count=count)
 
     def auto_up(self, count=1):
         """
@@ -316,24 +324,13 @@ class Line(object):
         if self.document.cursor_position_row < self.document.line_count - 1:
             self.cursor_position += self.document.get_cursor_down_position(count=count)
         elif not self.selection_state:
-            old_index = self._working_index
+            old_index = self.working_index
             self.history_forward(count=count)
 
             # If we moved to the next line, place the cursor at the beginning.
-            if old_index != self._working_index:
+            if old_index != self.working_index:
                 self.cursor_position = 0
 
-    def isearch_delete_before_cursor(self, count=1): # TODO: unittest return type
-        assert count >= 0
-
-        self._start_isearch()
-        self.isearch_state.isearch_text = self.isearch_state.isearch_text[:-count]
-
-        # When the `no_match_from_index` is after the character that we deleted. Remove this mark.
-        if (self.isearch_state.no_match_from_index is not None and
-                self.isearch_state.no_match_from_index >= len(self.isearch_state.isearch_text)):
-            self.isearch_state.no_match_from_index = None
-
     def delete_before_cursor(self, count=1): # TODO: unittest return type
         """ Delete character before cursor, return deleted character. """
         assert count >= 0
@@ -381,10 +378,10 @@ class Line(object):
         Go to this item in the history.
         """
         if index < len(self._working_lines):
-            self._working_index = index
+            self.working_index = index
             self.cursor_position = len(self.text)
 
-    def create_code_obj(self):
+    def create_code(self):
         """
         Create `Code` instance from the current input.
         """
@@ -402,7 +399,7 @@ class Line(object):
         Returns true if there was a completion.
         """
         # On the first tab press, try to find one completion and complete.
-        result = self.create_code_obj().get_common_complete_suffix()
+        result = self.create_code().get_common_complete_suffix()
         if result:
             self.text = self.insert_text(result)
             return True
@@ -448,7 +445,7 @@ class Line(object):
         Start completions. (Generate list of completions and initialize.)
         """
         # Generate list of all completions.
-        current_completions = list(self.create_code_obj().get_completions())
+        current_completions = list(self.create_code().get_completions())
 
         if current_completions:
             self.complete_state = CompletionState(
@@ -476,15 +473,15 @@ class Line(object):
         self.complete_state = state
 
     def history_forward(self, count=1):
-        if self._working_index < len(self._working_lines) - count:
+        if self.working_index < len(self._working_lines) - count:
             # Go forward in history, and update cursor_position.
-            self._working_index += count
+            self.working_index += count
             self.cursor_position = len(self.text)
 
     def history_backward(self, count=1):
-        if self._working_index - count >= 0:
+        if self.working_index - count >= 0:
             # Go back in history, and update cursor_position.
-            self._working_index -= count
+            self.working_index -= count
             self.cursor_position = len(self.text)
 
     def start_selection(self, selection_type=SelectionType.CHARACTERS):
@@ -547,10 +544,18 @@ class Line(object):
         self.cursor_position += self.document.get_end_of_line_position()
         self.insert_text(insert)
 
-    def insert_isearch_text(self, data):
-        self._start_isearch()
+    def set_search_text(self, text):
+        if not self.isearch_state:
+            self.start_isearch()
 
-        self.isearch_state.isearch_text += data
+        # When `text` is not a suffix of the search string that we had.
+        if not text.startswith(self.isearch_state.isearch_text):
+            self.isearch_state.no_match_from_index = None
+
+            self.working_index = self.isearch_state.original_working_index
+            self.cursor_position = self.isearch_state.original_cursor_position
+
+        self.isearch_state.isearch_text = text
 
         if not self.document.has_match_at_current_position(self.isearch_state.isearch_text):
             found = self.incremental_search(self.isearch_state.isearch_direction)
@@ -636,11 +641,11 @@ class Line(object):
         self.complete_state = None
         self._callbacks.exit()
 
-    def return_input(self):
+    def return_input(self, call_callback=True):
         """
         Return the current line to the `CommandLineInterface.read_input` call.
         """
-        code = self.create_code_obj()
+        code = self.create_code()
         text = self.text
 
         # Validate first. If not valid, set validation exception.
@@ -662,20 +667,26 @@ class Line(object):
                 self._history.append(text)
 
         # Callback
-        self._callbacks.return_input(code)
+        if call_callback:
+            self._callbacks.return_input(code)
 
-    def _start_isearch(self):
-        if not self.isearch_state:
-            self.isearch_state = _IncrementalSearchState(
-                    original_cursor_position = self.cursor_position,
-                    original_working_index = self._working_index)
+    def start_isearch(self, direction=IncrementalSearchDirection.FORWARD):
+        """
+        Start incremental search.
+        Take the current position as the start position for the search.
+        """
+        self.isearch_state = _IncrementalSearchState(
+                original_cursor_position = self.cursor_position,
+                original_working_index = self.working_index,
+                direction=direction)
 
     def incremental_search(self, direction):
         """
         Search for the next string.
         :returns: (bool) True if something was found.
         """
-        self._start_isearch()
+        if not self.isearch_state:
+            self.start_isearch()
 
         found = False
         self.isearch_state.isearch_direction = direction
@@ -691,11 +702,11 @@ class Line(object):
                 found = True
             else:
                 # No match, go back in the history.
-                for i in range(self._working_index - 1, -1, -1):
+                for i in range(self.working_index - 1, -1, -1):
                     document = Document(self._working_lines[i], len(self._working_lines[i]))
                     new_index = document.find_backwards(isearch_text)
                     if new_index is not None:
-                        self._working_index = i
+                        self.working_index = i
                         self.cursor_position = len(self._working_lines[i]) + new_index
                         self.isearch_state.no_match_from_index = None
                         found = True
@@ -709,11 +720,11 @@ class Line(object):
                 found = True
             else:
                 # No match, go forward in the history.
-                for i in range(self._working_index + 1, len(self._working_lines)):
+                for i in range(self.working_index + 1, len(self._working_lines)):
                     document = Document(self._working_lines[i], 0)
                     new_index = document.find(isearch_text, include_current_position=True)
                     if new_index is not None:
-                        self._working_index = i
+                        self.working_index = i
                         self.cursor_position = new_index
                         self.isearch_state.no_match_from_index = None
                         found = True
@@ -728,8 +739,8 @@ class Line(object):
         """
         Exit i-search mode.
         """
-        if restore_original_line:
-            self._working_index = self.isearch_state.original_working_index
+        if restore_original_line and self.isearch_state:
+            self.working_index = self.isearch_state.original_working_index
             self.cursor_position = self.isearch_state.original_cursor_position
 
         self.isearch_state = None
